package org.jconfig.visual;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import java.text.*;
import java.lang.*;
import org.jconfig.items.*;

/**
 *  <p>package:    org.jconfig.visual</p>
 *  <p>file:       ConfigNumField.java</p>
 * $Id: ConfigNumField.java,v 1.3 2004/04/19 08:12:45 tgl Exp $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 *	Mumber field widget for configuration editor
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class ConfigNumField extends JFormattedTextField {
    
    public  String   id, min, max;
    public  TreeItem item;
    private int      type;
    
    
    /** Creates a new instance of ConfigTextArea */
    public ConfigNumField(TreeItem it, TreeItem si, String name, boolean ismask, int ulevel) {
        item = it;
        id = name;

        TreeItem fmtit = null;
        for (int i = 0; i < si.items.size(); i++) {
            fmtit = (TreeItem) si.items.get(i);
            if (fmtit.HasName("formatitem"))
                break;
            else
                fmtit = null;
        }
        
        String format = null;
        String mask = null;
        String def = null;
        String regex = null;
        if (fmtit != null) {
            format = fmtit.GetAttrValue("format");
            mask = fmtit.GetAttrValue("mask");
            regex = fmtit.GetAttrValue("regex");
            def = fmtit.GetAttrValue("default");
            min = fmtit.GetAttrValue("min");
            max = fmtit.GetAttrValue("max");
            try {
                type = Integer.parseInt(fmtit.GetAttrValue("type"));
            } catch ( Exception e) { type = 0; }
        }

        if (def == null)
            def = "0";
        
        if (!ismask)
            SetNumberFormat (format, regex);
        else 
            SetMaskFormat (format, regex, mask);

        setValue(it.GetOrBuildAttrValue(id, def));

        this.setEnabled(ConfigPanel.ConfigCheckUserlevel(it, ulevel));
        
        String tip = "";
        if (def != null)
            tip += "Default: " + def + " ";
        if (min != null && min.length() > 0)
            tip += "Min: " + min + " ";
        if (max != null && max.length() > 0)
            tip += "Max: " + max + " ";
            
        this.setToolTipText(tip);
            
        addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {}
            
            public void focusLost(FocusEvent e) {
                if (isEditValid()) {
                    String txt = getText();
                    try {
                    switch (type) {
                        // Decimal format
                        case 0: {
                            double val = Double.parseDouble(txt);
                            if (min != null && min.length() > 0) {
                                double cmp = Double.parseDouble(min);
                                if (val < cmp) {
                                    setValue(item.GetAttrValue(id));
                                    return;
                                }    
                            }
                            if (max != null && max.length() > 0) {
                                double cmp = Double.parseDouble(max);
                                if (val > cmp) {
                                    setValue(item.GetAttrValue(id));
                                    return;
                                }    
                            }
                            break;
                        }
                        // Hexadecimal format
                        case 1: {
                            int val = Integer.parseInt(txt, 16);
                            if (min != null && min.length() > 0) {
                                int cmp = Integer.parseInt(min, 16);
                                if (val < cmp) {
                                    setValue(item.GetAttrValue(id));
                                    return;
                                }    
                            }
                            if (max != null && max.length() > 0) {
                                int cmp = Integer.parseInt(max, 16);
                                if (val > cmp) {
                                    setValue(item.GetAttrValue(id));
                                    return;
                                }    
                            }
                            break;
                        }
                    }
                    } catch (Exception ex) { return; }
                    item.ModifyAttr(id, txt);
                    System.out.println (txt);
                }
            }
        });
    }

    public void SetNumberFormat (String format, String regex) {
        
        if (regex != null && regex.length() > 0) {
            SetRegexFormat(regex);
            return;
        }
        if (format != null && format.length() > 0) {
            DefaultFormatter fmt;
            fmt = new NumberFormatter(new DecimalFormat(format));
            fmt.setAllowsInvalid(false);
            DefaultFormatterFactory fmtFactory = new DefaultFormatterFactory(fmt);
            setFormatterFactory(fmtFactory);
        }
        SetRegexFormat("-?+\\d{1,9}");
    }
    
    private void SetMaskFormat (String format, String regex, String mask) {
        if (format == null && regex == null) {
            SetDefaultFormat ();
            return;
        }
        if (regex.length() > 0) {
            SetRegexFormat(regex);
            return;
        }
        MaskFormatter fmt;
        try {
            fmt = new MaskFormatter (format);
            if (mask != null && mask.length() > 0)
                fmt.setValidCharacters(mask);
            fmt.setAllowsInvalid(false);
            DefaultFormatterFactory fmtFactory = new DefaultFormatterFactory(fmt);
            setFormatterFactory(fmtFactory);
        } catch (Exception e) {
            SetDefaultFormat();
        }
    }
    
    private void SetDefaultFormat() {
        setFormatterFactory (new DefaultFormatterFactory (new DefaultFormatter()));    
    }
    
    private void SetRegexFormat (String format) {
        ConfigRegexFormatter fmt;
        fmt = new ConfigRegexFormatter (format);
        DefaultFormatterFactory fmtFactory = new DefaultFormatterFactory(fmt);
        setFormatterFactory(fmtFactory);
    }
   
}
