package org.jconfig.visual;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import org.jconfig.items.*;

/**
 *  <p>package:    org.jconfig.visual</p>
 *  <p>file:       ConfigPanel.java</p>
 * $Id: ConfigPanel.java,v 1.2 2004/04/19 08:12:45 tgl Exp $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 *	Panel to display a configuration set
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class ConfigPanel extends JPanel {
    
    private boolean ParamMode;
    private int     UserLevel;
    
    /** Creates a new instance of ConfigPanel */
    public ConfigPanel (boolean parammode) {
        ParamMode = parammode;
    }
    
    public ConfigPanel(TreeItem it, boolean parammode, int axis, int userlevel) {
        ParamMode = parammode;
        SetPanelOptions(axis);
        SetConfigItems (it, false, userlevel);
    }

    public ConfigPanel(boolean parammode, int axis) {
        ParamMode = parammode;
        SetPanelOptions(axis);
    }
    
    public void SetPanelOptions (int axis) {
        // Setup the panel
        setLayout(new BoxLayout(this, axis));
        setAlignmentX(LEFT_ALIGNMENT);
    }
    
    public void SetConfigItems (TreeItem item, boolean restore, int userlevel) {
        int i;
        String name, typ;
        TreeItem cfg = ParamMode ? item : item.GetConfig();
        UserLevel = userlevel;
        
        removeAll ();
        repaint ();
                
        for (i = 0; i < cfg.items.size(); i++) {
            // Read subitem
            TreeItem si = (TreeItem) cfg.items.get(i);
            // Get name and type    
            name = ParamMode ? "value" : si.GetAttrValue("name");
            TreeItem it = ParamMode ? si : item;
            typ = si.GetAttrValue("typ");
            if (typ == null || name==null)
                continue;
            // Check for TextArea
            if (CfgTextArea (it, si, name, typ) == true)
                continue;
            // Check for CheckBox
            if (CfgCheckBox (it, si, name, typ) == true)
                continue;
            // Check for TextField
            if (CfgTextField (it, si, name, typ) == true)
                continue;
            // Check for ComboBox
            if (CfgComboBox (it, si, name, typ) == true)
                continue;
            // Check for Numberfield
            if (CfgNumField (it, si, name, typ) == true)
                continue;
            // Check for Maskfield
            if (CfgMaskField (it, si, name, typ) == true)
                continue;
            // Check for Passwordfield
            if (CfgPasswField (it, si, name, typ) == true)
                continue;
            // Check for Cronjobfield
            if (CfgCronJField (it, si, name, typ) == true)
                continue;
            // Check for a hidden field, which passes a fixed value
            if (CfgHiddenField (it, si, name, typ) == true)
                continue;
        }
        revalidate ();
    }

    private void addTo (Component c, int h) {
        addTo (c, null, h);
    }
    
    private void addTo (Component c, String label, int h) {
        JPanel j = new JPanel();
        j.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED));
        j.setAlignmentX(LEFT_ALIGNMENT);
        j.setLayout(new BoxLayout(j, BoxLayout.Y_AXIS));
        Dimension dim = new Dimension (this.getParent().getWidth(), h);
        j.setPreferredSize (dim);
        j.setMinimumSize(dim);
        j.setMaximumSize(dim);
        if (label != null)
            j.add (new JLabel(label));
        j.add (c);
        add (j);
    }

    private boolean CfgHiddenField (TreeItem it, TreeItem si, String name, String type) {
        if (type.compareTo("Hidden") != 0)
            return false;
        it.ModifyAttr(name, si.GetAttrValue("value"));
        return true;
    }

    private boolean CfgCronJField (TreeItem it, TreeItem si, String name, String type) {
        if (type.compareTo("CronJobs") != 0)
            return false;
        ConfigCronJobs tf = new ConfigCronJobs(it, si, name, UserLevel);
        addTo (tf, si.toString(), 200);
        return true;
    }

    private boolean CfgPasswField (TreeItem it, TreeItem si, String name, String type) {
        if (type.compareTo("PasswField") != 0)
            return false;
        ConfigPasswField tf = new ConfigPasswField(it, name, UserLevel);
        addTo (tf, si.toString(), 50);
        return true;
    }
    
    private boolean CfgMaskField (TreeItem it, TreeItem si, String name, String type) {
        if (type.compareTo("MaskField") != 0)
            return false;
        ConfigNumField tf = new ConfigNumField(it, si, name, true, UserLevel);
        addTo (tf, si.toString(), 50);
        return true;
    }

    private boolean CfgNumField (TreeItem it, TreeItem si, String name, String type) {
        if (type.compareTo("NumField") != 0)
            return false;
        ConfigNumField tf = new ConfigNumField(it, si, name, false, UserLevel);
        addTo (tf, si.toString(), 50);
        return true;
    }
    
    private boolean CfgComboBox (TreeItem it, TreeItem si, String name, String type) {
        if (type.compareTo("ComboBox") != 0)
            return false;
        ConfigComboBox cb = new ConfigComboBox(it, si, name, UserLevel);
        
        addTo (cb, si.toString(), 50);
        return true;
    }
    
    private boolean CfgTextField (TreeItem it, TreeItem si, String name, String type) {
        if (type.compareTo("TextField") != 0)
            return false;
        ConfigTextField tf = new ConfigTextField(it, si, name, UserLevel);
        
        addTo (tf, si.toString(), 50);
        return true;
    }

    private boolean CfgCheckBox (TreeItem it, TreeItem si, String name, String type) {
        if (type.compareTo("CheckBox") != 0)
            return false;
        
        ConfigCheckBox cb = new ConfigCheckBox(si.toString(), it, name, UserLevel);
        
        addTo (cb, 50);
        return true;
    }
    
    private boolean CfgTextArea (TreeItem it, TreeItem si, String name, String type) {
        
        if (type.compareTo("TextArea") != 0)
            return false;
        
        int col = 20, row = 3;
        
        try { col = Integer.parseInt(si.GetAttrValue("col")); }
        catch (Exception e) { }
        try { row = Integer.parseInt(si.GetAttrValue("row")); }
        catch (Exception e) { }
        
        ConfigTextArea ta = new ConfigTextArea(row, col, it, name.compareTo("text") == 0 ? true : false, name, UserLevel);

        addTo (ta, si.toString(), 50 + row * 10);
        return true;
    }
    
    public static boolean ConfigCheckUserlevel(TreeItem it, int ulevel) {
        String val = it.GetAttrValue("userlevel");
        if (val == null || ulevel == -1)
            return true;
        try {
            int lvl = Integer.parseInt(val);
            return (ulevel >= lvl);
        } catch (NumberFormatException e) {}
        return false;
    }
}
