/*
 * JVCommCfg.java
 *
 * Created on 27. Oktober 2003, 08:33
 */

package org.jvisu.comm;

import java.lang.Object;

/** Configuration storage for JVisu communication.
 *
 *  <p>package:  org.comm.var</p>
 *  <p>file:    JVCommCfg.java</p>
 *  <p>$Id: JVCommCfg.java,v 1.5 2004/04/19 08:20:55 tgl Exp $ </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *  
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p>
 *  Store all communication related configuation information
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0 
*/

public class JVCommCfg extends Object{
    
    /** Communication type */
    private String      type;
    /** Communication server */
    private String      server;
    /** Communication port */
    private int         port;
    /** Communication resolver */
    private String      resolver;
    /** The communication instance, which is associated 
     * to this configuration 
     */
    private JVComm      commInst;
    
    /** Creates a new instance of JVCommCfg.
     *  Default the type to simulation 
     */
    public JVCommCfg() {
        type = "org.jvisu.comm.JVSimComm";
    }

    /** Creates a new instance of JVCommCfg.
     *  Copy the propertuies from a given instance
     */
    public JVCommCfg(JVCommCfg cfg) {
        type = cfg.getType();
        server = cfg.getServer();
        port = cfg.getPort();
        resolver = cfg.getResolver();
    }
    
    /** Return the communication type
     *  @return communication type
     */
    public String getType() {
        return this.type;
    }    
    
    /** Set the communication type
     * @param type New communication type.
     */
    public void setType(String type) {
        this.type = type;
    }
    
    /** Return the server name
     * @return server name.
     *
     */
    public String getServer() {
        return this.server;
    }
    
    /** Set the server name
     * @param server new server name.
     */
    public void setServer(String server) {
        this.server = server;
    }
    
    /** Return the class name of the communication resolver
     * @return resolver class name.
     */
    public String getResolver() {
        return this.resolver;
    }
    
    /** Set the class name of the communication resolver
     * @param resolv new resolver class name.
     */
    public void setResolver(String resolv) {
        resolver = resolv;
    }

    /** Return the communciation port.
     * @return communicaiton port.
     */
    public int getPort() {
        return this.port;
    }
    
    /** Set the communicatio port.
     * @param port new communication port.
     */
    public void setPort(int port) {
        this.port = port;
    }

    /** Get the communication instance, which is associated with this 
     *  configuration
     *  @return communication instance
     */
    public JVComm getCommInstance() {
        return commInst;
    }

    /** Set the communication instance, which is associated with this 
     *  configuration
     *  @param inst communication instance
     */
    public void setCommInstance(JVComm inst) {
        commInst = inst;
    }
}
