package org.jvisu.panel.configurator;

import java.io.*;
import java.awt.*;
import java.lang.reflect.*;
import java.util.*;
import java.beans.*;
import javax.swing.*;

/**
 *  <p>package:    org.jvisu.panel</p>
 *  <p>file:       JVPanelTreeToXML.java</p>
 * $Id: JVPanelTreeToXML.java,v 1.3 2004/04/19 08:20:55 tgl Exp $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>g
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 * Convert the component tree to a XML representation
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVPanelTreeToXML {
    
    private int             level;
    private StringWriter    xml;
    
    /** Create a new instance 
     */
    public JVPanelTreeToXML() {
        level = 0;
        xml = new StringWriter();
        xml.write ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
    }
    
    /** Print indentation
     */
    private void PrintIndent () {
        xml.write('\n');
        for (int i=0; i < level; i++)
            xml.write('\t');
    }
    
    /** Encode the string UTF-8
     *  @param content  the content, which must be encoded
     *  @return         the encoded content
     */
    private String Encode (String content) {
        try {
            return java.net.URLEncoder.encode(content, "UTF-8");
        } catch (Exception e) { 
            System.out.println(content);
        }
        return content;
    }
    
    /* Write the start tag of an element
     *  @param tagname the qualified name of this tag
     */
    public void writeTagStart (String tagname) {
        PrintIndent();
        level++;
        xml.write("<" + tagname);
    }
    
    /* Write attribute
     *  @param name attribute name
     *  @param value attribute value, which will be UTF-8 encoded
     */
    public void writeTagAttribute(String name, String value) {
        xml.write (" " + name + "=\"" + Encode(value) + "\"");
    }

    /* Write end tag of the element
     *  @param isEnd true, if this is a <xyz att="zxy"/> style tag
     */
    public void writeTagEnd(boolean isEnd) {
        if (isEnd) {
            xml.write ("/>");
            level--;
        }
        else
            xml.write (">");
    }
    
    /* Write the content of the element
     *  @param content content, which will be UTF-8 encoded
     */
    public void writeContent (String content) {
        xml.write(Encode(content));
    }
    
    /*  Write the end tag of the element
     *  @param tagname the qualified name of this tag
     *  @param mustindent true, if this tag is in a seperate line. false
     *      for single line elements
     */
    public void writeEndTag (String tagname, boolean mustindent) {
        level--;
        if (mustindent)
            PrintIndent();
        xml.write ("</" + tagname + ">");
    }

    /*  Write a complete tag with indentation
     *  @param tag the complete tag is written
     */
    public void writeTag (String tag) {
        PrintIndent();
        xml.write(tag);
    }
    
    /*  Return the string representation of this xml file
     *  @return the content, which was written 
     */
    public String GetString() {
        xml.write("\n");
        return xml.toString();
    }
    
    /** Get the XML representation of the components properties,
     *  which differ from the default value.
     *  @param comp component to retrieve properties
     *  @param ref  reference component (default values)
     *  @param constraints ArrayList containing palette constraints
     */
    public void componentPropertiesToXML (JComponent comp, JComponent ref, ArrayList constraints) {

        if (comp == null || ref == null || constraints == null)
            return;
        
        for (int i = 0; i < constraints.size(); i++) {
            JVPalettePanel.PaletteItemProperty prop = 
                    (JVPalettePanel.PaletteItemProperty) constraints.get(i);
            if (prop == null)
                continue;
			if ( prop.isExpert())
				continue;	
            if ( prop.getCategory() != null && prop.getCategory().equalsIgnoreCase("designtime"))
                continue;
            
            String meth = prop.getName().toUpperCase().substring(0,1) +
                          prop.getName().substring(1);
        
            Method readm = null;
            try {
                readm = comp.getClass().getMethod(("get" + meth), null);
            } catch (Exception e) {
                try {
                    readm = comp.getClass().getMethod(("is" + meth), null);
                } catch (Exception e1) {
                    continue;
                }
            }
            Object obj1, obj2;
            try {
                obj1 = readm.invoke(comp, null);
                obj2 = readm.invoke(ref, null);
                if (obj1 == null && obj2 == null)
                    continue;
                if (obj1 != null && obj1.equals(obj2))
                    continue;
            } catch (Exception e) {
                continue;
            }
            
            /* Convert the object to a String representation */
            String txt = org.jvisu.util.param.JVParams.ObjectToString(obj1);
            
            writeTagStart("property");
            writeTagAttribute("name", prop.getName());
            writeTagAttribute("value", txt);
            writeTagEnd(true);
        }
    }
}
